﻿using System;
using SDX.Compiler;
using Mono.Cecil;
using Mono.Cecil.Cil;
using System.Linq;
using System.Collections.Generic;

public class SightFix : IPatcherMod
{
    public bool Link(ModuleDefinition gameModule, ModuleDefinition modModule)
    {
        InsertInitEntityClassProp(gameModule, modModule);
        InsertCopyPropertiesFromEntityClass(gameModule, modModule);
        FixGetSeeDistance(gameModule, modModule);
        FixCanSeeStealth(gameModule, modModule);
        FixGetSleeperSightScale(gameModule, modModule);
        FixGetSeeStealthDebugScale(gameModule, modModule);
        FixGetSleeperDebugScale(gameModule, modModule);

        

        return true;
    }
    public void InsertInitEntityClassProp(ModuleDefinition gameModule, ModuleDefinition modModule)
    {
        TypeDefinition entityClass = gameModule.Types.First(d => d.Name == "EntityClass");
        MethodDefinition init = entityClass.Methods.First(d => d.Name == "Init");

        int state = 0;
        Instruction instBr = null;
        foreach (Instruction inst in init.Body.Instructions)
        {
            switch (state)
            {
                case 0:
                    if (inst.OpCode == OpCodes.Ldsfld && inst.Operand.GetType() == typeof(FieldDefinition) && ((FieldDefinition)inst.Operand).Name == "PropSightRange")
                    {
                        state = 1;

                    }
                    break;
                case 1:
                    if (inst.OpCode == OpCodes.Brfalse_S)
                    {
                        instBr = inst;
                        state = 2;
                    }
                    break;
            }
        }

        Instruction insertPos = (Instruction)instBr.Operand;

        ILProcessor proc1 = init.Body.GetILProcessor();
        Instruction newJumpPos = Instruction.Create(OpCodes.Ldarg_0);
        proc1.InsertBefore(insertPos, newJumpPos);





        TypeDefinition sightFixMethod = modModule.Types.First(d => d.Name == "SightFixMethod");
        MethodDefinition initEntityClassProp = sightFixMethod.Methods.First(d => d.Name == "InitEntityClassProp");
        MethodReference initEntityClassPropRef = gameModule.Import(initEntityClassProp);


        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Call, initEntityClassPropRef));


        instBr.Operand = newJumpPos;

    }
    public void InsertCopyPropertiesFromEntityClass(ModuleDefinition gameModule, ModuleDefinition modModule)
    {
        
        TypeDefinition entityAlive = gameModule.Types.First(d => d.Name == "EntityAlive");
        MethodDefinition copyPropertiesFromEntityClass = entityAlive.Methods.First(d => d.Name == "CopyPropertiesFromEntityClass");

        int state = 0;
        Instruction insertPos = null;
        foreach (Instruction inst in copyPropertiesFromEntityClass.Body.Instructions)
        {
            switch (state)
            {
                case 0:
                    if (inst.OpCode == OpCodes.Stfld && inst.Operand.GetType() == typeof(FieldDefinition) && ((FieldDefinition)inst.Operand).Name == "IR")
                    {
                        state = 1;
                        insertPos = inst.Next;

                    }
                    break;
            }
        }


        ILProcessor proc1 = copyPropertiesFromEntityClass.Body.GetILProcessor();
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Ldarg_0));
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Ldloc_0));
        

        TypeDefinition sightFixMethod = modModule.Types.First(d => d.Name == "SightFixMethod");
        MethodDefinition copyPropertiesFromEntityClassAdd = sightFixMethod.Methods.First(d => d.Name == "CopyPropertiesFromEntityClassAdd");
        MethodReference copyPropertiesFromEntityClassAddRef = gameModule.Import(copyPropertiesFromEntityClassAdd);


        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Call, copyPropertiesFromEntityClassAddRef));


    }
    public void FixGetSeeDistance(ModuleDefinition gameModule, ModuleDefinition modModule)
    {
        TypeDefinition entityAlive = gameModule.Types.First(d => d.Name == "EntityAlive");
        MethodDefinition getSeeDistance = entityAlive.Methods.First(d => d.Name == "GetSeeDistance");

        int state = 0;
        Instruction insertPos = null;
        foreach (Instruction inst in getSeeDistance.Body.Instructions)
        {
            switch (state)
            {
                case 0:
                    if (inst.OpCode == OpCodes.Ldfld && inst.Operand.GetType() == typeof(FieldDefinition) && ((FieldDefinition)inst.Operand).Name == "IR")
                    {
                        state = 1;
                        insertPos = inst;

                    }
                    break;
            }
        }


        ILProcessor proc1 = getSeeDistance.Body.GetILProcessor();


        TypeDefinition sightFixMethod = modModule.Types.First(d => d.Name == "SightFixMethod");
        MethodDefinition getSightRange = sightFixMethod.Methods.First(d => d.Name == "GetSightRange");
        MethodReference getSightRangeRef = gameModule.Import(getSightRange);


        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Call, getSightRangeRef));

        proc1.Remove(insertPos);



    }
    public void FixCanSeeStealth(ModuleDefinition gameModule, ModuleDefinition modModule)
    {
        TypeDefinition entityAlive = gameModule.Types.First(d => d.Name == "EntityAlive");
        MethodDefinition canSeeStealth = entityAlive.Methods.First(d => d.Name == "CanSeeStealth");

        int state = 0;
        Instruction insertPos = canSeeStealth.Body.Instructions[0];
        Instruction lightPos = null;
        List<Instruction> ldfldIRList = new List<Instruction>();
        foreach (Instruction inst in canSeeStealth.Body.Instructions)
        {
            switch (state)
            {
                case 0:
                    if (inst.OpCode == OpCodes.Ldarg_2)
                    {
                        lightPos = inst;
                        state = 1;
                    }
                    break;
            }
            if (inst.OpCode == OpCodes.Ldfld && inst.Operand.GetType() == typeof(FieldDefinition) && ((FieldDefinition)inst.Operand).Name == "IR")
            {
                ldfldIRList.Add(inst);

            }
        }


        ILProcessor proc1 = canSeeStealth.Body.GetILProcessor();

        VariableDefinition vd = new VariableDefinition("V_3", gameModule.Import(typeof(float)));
        proc1.Body.Variables.Add(vd);


        TypeDefinition sightFixMethod = modModule.Types.First(d => d.Name == "SightFixMethod");
        MethodDefinition getSightRange = sightFixMethod.Methods.First(d => d.Name == "GetSightRange");
        MethodReference getSightRangeRef = gameModule.Import(getSightRange);


        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Ldarg_0));
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Call, getSightRangeRef));
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Stloc_3));

        foreach (Instruction inst in ldfldIRList)
        {
            proc1.Remove(inst.Previous);
            proc1.InsertBefore(inst, Instruction.Create(OpCodes.Ldloc_3));
            proc1.Remove(inst);

        }

        MethodDefinition getLightLevel = sightFixMethod.Methods.First(d => d.Name == "GetLightLevel");
        MethodReference getLightLevelRef = gameModule.Import(getLightLevel);
        proc1.InsertBefore(lightPos, Instruction.Create(OpCodes.Ldarg_0));
        proc1.InsertBefore(lightPos.Next, Instruction.Create(OpCodes.Call, getLightLevelRef));

    }
    public void FixGetSleeperSightScale(ModuleDefinition gameModule, ModuleDefinition modModule)
    {
        TypeDefinition entityAlive = gameModule.Types.First(d => d.Name == "EntityAlive");
        MethodDefinition getSleeperSightScale = entityAlive.Methods.First(d => d.Name == "GetSleeperSightScale");

        int state = 0;
        Instruction insertPos = getSleeperSightScale.Body.Instructions[0];
        List<Instruction> ldfldIRList = new List<Instruction>();
        List<Instruction> lightPosList = new List<Instruction>();
        foreach (Instruction inst in getSleeperSightScale.Body.Instructions)
        {
            if (inst.OpCode == OpCodes.Ldarg_2)
            {
                lightPosList.Add(inst);
                state = 1;
            }
            if (inst.OpCode == OpCodes.Ldfld && inst.Operand.GetType() == typeof(FieldDefinition) && ((FieldDefinition)inst.Operand).Name == "IR")
            {
                ldfldIRList.Add(inst);

            }
        }


        ILProcessor proc1 = getSleeperSightScale.Body.GetILProcessor();

        VariableDefinition vd = new VariableDefinition("V_4", gameModule.Import(typeof(float)));
        proc1.Body.Variables.Add(vd);
        VariableDefinition vd2 = new VariableDefinition("V_5", gameModule.Import(typeof(float)));
        proc1.Body.Variables.Add(vd2);


        TypeDefinition sightFixMethod = modModule.Types.First(d => d.Name == "SightFixMethod");
        MethodDefinition getSightRange = sightFixMethod.Methods.First(d => d.Name == "GetSightRange");
        MethodReference getSightRangeRef = gameModule.Import(getSightRange);


        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Ldarg_0));
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Call, getSightRangeRef));
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Stloc_S, vd));

        MethodDefinition getLightLevel = sightFixMethod.Methods.First(d => d.Name == "GetLightLevel");
        MethodReference getLightLevelRef = gameModule.Import(getLightLevel);

        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Ldarg_0));
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Ldarg_2));
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Call, getLightLevelRef));
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Stloc_S, vd2));

        foreach (Instruction inst in ldfldIRList)
        {
            proc1.Remove(inst.Previous);
            proc1.InsertBefore(inst, Instruction.Create(OpCodes.Ldloc_S, vd));
            proc1.Remove(inst);

        }

        foreach (Instruction inst in lightPosList)
        {
//            proc1.Remove(inst.Previous);
            proc1.InsertBefore(inst, Instruction.Create(OpCodes.Ldloc_S, vd2));
            proc1.Remove(inst);

        }

    }
    public void FixGetSeeStealthDebugScale(ModuleDefinition gameModule, ModuleDefinition modModule)
    {
        TypeDefinition entityAlive = gameModule.Types.First(d => d.Name == "EntityAlive");
        MethodDefinition getSeeStealthDebugScale = entityAlive.Methods.First(d => d.Name == "GetSeeStealthDebugScale");

        Instruction insertPos = getSeeStealthDebugScale.Body.Instructions[0];
        List<Instruction> ldfldIRList = new List<Instruction>();
        foreach (Instruction inst in getSeeStealthDebugScale.Body.Instructions)
        {
            if (inst.OpCode == OpCodes.Ldfld && inst.Operand.GetType() == typeof(FieldDefinition) && ((FieldDefinition)inst.Operand).Name == "IR")
            {
                ldfldIRList.Add(inst);

            }
        }


        ILProcessor proc1 = getSeeStealthDebugScale.Body.GetILProcessor();

        VariableDefinition vd = new VariableDefinition("V_2", gameModule.Import(typeof(float)));
        proc1.Body.Variables.Add(vd);


        TypeDefinition sightFixMethod = modModule.Types.First(d => d.Name == "SightFixMethod");
        MethodDefinition getSightRange = sightFixMethod.Methods.First(d => d.Name == "GetSightRange");
        MethodReference getSightRangeRef = gameModule.Import(getSightRange);


        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Ldarg_0));
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Call, getSightRangeRef));
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Stloc_2));

        foreach (Instruction inst in ldfldIRList)
        {
            proc1.Remove(inst.Previous);
            proc1.InsertBefore(inst, Instruction.Create(OpCodes.Ldloc_2));
            proc1.Remove(inst);

        }
    }
    public void FixGetSleeperDebugScale(ModuleDefinition gameModule, ModuleDefinition modModule)
    {
        TypeDefinition entityAlive = gameModule.Types.First(d => d.Name == "EntityAlive");
        MethodDefinition getSleeperDebugScale = entityAlive.Methods.First(d => d.Name == "GetSleeperDebugScale");

        Instruction insertPos = getSleeperDebugScale.Body.Instructions[0];
        List<Instruction> ldfldIRList = new List<Instruction>();
        foreach (Instruction inst in getSleeperDebugScale.Body.Instructions)
        {
            if (inst.OpCode == OpCodes.Ldfld && inst.Operand.GetType() == typeof(FieldDefinition) && ((FieldDefinition)inst.Operand).Name == "IR")
            {
                ldfldIRList.Add(inst);

            }
        }


        ILProcessor proc1 = getSleeperDebugScale.Body.GetILProcessor();

        VariableDefinition vd = new VariableDefinition("V_2", gameModule.Import(typeof(float)));
        proc1.Body.Variables.Add(vd);


        TypeDefinition sightFixMethod = modModule.Types.First(d => d.Name == "SightFixMethod");
        MethodDefinition getSightRange = sightFixMethod.Methods.First(d => d.Name == "GetSightRange");
        MethodReference getSightRangeRef = gameModule.Import(getSightRange);


        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Ldarg_0));
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Call, getSightRangeRef));
        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Stloc_2));

        foreach (Instruction inst in ldfldIRList)
        {
            proc1.Remove(inst.Previous);
            proc1.InsertBefore(inst, Instruction.Create(OpCodes.Ldloc_2));
            proc1.Remove(inst);

        }
    }
    


    public bool Patch(ModuleDefinition module)
    {
        Console.WriteLine("==SightFix Patcher===");
        AddEntityClassField(module);
        AddEntityAliveField(module);
        EntityAliveIRFieldPublic(module);
        return true;
    }
    public void AddEntityClassField(ModuleDefinition module)
    {
        TypeDefinition entityClass = module.Types.First(d => d.Name == "EntityClass");
        TypeReference tr = module.Import(typeof(float));
        AddFieldDefinition(entityClass, "DarkSight", FieldAttributes.Public , tr);
        AddFieldDefinition(entityClass, "NightDarkSight", FieldAttributes.Public , tr);
        AddFieldDefinition(entityClass, "NightSightRange", FieldAttributes.Public , tr);

    }
    public void AddEntityAliveField(ModuleDefinition module)
    {
        TypeDefinition entityAlive = module.Types.First(d => d.Name == "EntityAlive");
        TypeReference tr = module.Import(typeof(float));
        AddFieldDefinition(entityAlive, "DarkSight", FieldAttributes.Public , tr);
        AddFieldDefinition(entityAlive, "NightDarkSight", FieldAttributes.Public , tr);
        AddFieldDefinition(entityAlive, "NightSightRange", FieldAttributes.Public , tr);

    }
    public void EntityAliveIRFieldPublic(ModuleDefinition module)
    {
        TypeDefinition entityAlive = module.Types.First(d => d.Name == "EntityAlive");
        FieldDefinition field = entityAlive.Fields.First(d => d.Name == "IR");
        SetFieldToPublic(field);
    }

    public void AddFieldDefinition(TypeDefinition classDef, string fieldName, FieldAttributes fieldAttribute, TypeReference fieldType)
    {
        FieldDefinition fd = new FieldDefinition(fieldName, fieldAttribute, fieldType);
        classDef.Fields.Add(fd);

    }
    public void AddFieldDefinition(TypeDefinition classDef, string fieldName, FieldAttributes fieldAttribute, TypeReference fieldType, object constantvalue)
    {
        FieldDefinition fd = new FieldDefinition(fieldName, fieldAttribute, fieldType);
        fd.Constant = constantvalue;
        classDef.Fields.Add(fd);

    }
    private void SetFieldToPublic(FieldDefinition field)
    {
        field.IsFamily = false;
        field.IsPrivate = false;
        field.IsPublic = true;

    }

}
