﻿using System;
using System.Linq;
using System.Xml;
using System.Collections.Generic;
using UnityEngine;

public static class SightFixMethod
{
    public static float GetSightRange(EntityAlive entityAlive)
    {
        if (!entityAlive.isFeral && !entityAlive.world.IsDark())
        {
            return entityAlive.IR;
        }
        return entityAlive.NightSightRange;
    }
    public static float GetDarkSight(EntityAlive entityAlive)
    {
        if (!entityAlive.isFeral && !entityAlive.world.IsDark())
        {
            return entityAlive.DarkSight;
        }
        return entityAlive.NightDarkSight;
    }
    public static float GetLightLevel(EntityAlive entityAlive, float lightLevel)
    {
        return Mathf.Clamp(lightLevel + GetDarkSight(entityAlive), 0f, 200f);
    }
    
    public static void InitEntityClassProp(EntityClass entityClass)
    {
        if (entityClass.Properties.Values.ContainsKey("NightSightRange"))
        {
            entityClass.NightSightRange = Utils.ParseFloat(entityClass.Properties.Values["NightSightRange"]);
        }
        else
        {
            entityClass.NightSightRange = Constants.cDefaultMonsterSeeDistance;
        }
        if (entityClass.Properties.Values.ContainsKey("DarkSight"))
        {
            entityClass.DarkSight = Utils.ParseFloat(entityClass.Properties.Values["DarkSight"]);
        }
        else
        {
            entityClass.DarkSight = 0f;
        }
        if (entityClass.Properties.Values.ContainsKey("NightDarkSight"))
        {
            entityClass.NightDarkSight = Utils.ParseFloat(entityClass.Properties.Values["NightDarkSight"]);
        }
        else
        {
            entityClass.NightDarkSight = 0f;
        }
    }
    public static void CopyPropertiesFromEntityClassAdd(EntityAlive entityAlive,EntityClass entityClass)
    {
        entityAlive.NightSightRange = entityClass.NightSightRange;
        entityAlive.DarkSight = entityClass.DarkSight;
        entityAlive.NightDarkSight = entityClass.NightDarkSight;
    }

}
